local bump       = require 'bump'
local bump_debug = require 'bump_debug'

--*******************************************
--*******************************************
--*******************************************
--           load
--*******************************************
--*******************************************
--*******************************************

function love.load()
    gr = love.graphics
    kb = love.keyboard
    mo = love.mouse
    to = love.touch
    ma = love.math

    win = {
        w = gr.getWidth(),
        h = gr.getHeight()
    }
    map1 = {
        {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,},
        {0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0,},
        {0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0,},
        {0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 0,},
        {0, 1, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 0,},
        {0, 1, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 0,},
        {0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0,},
        {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,},
    }

    canvasW = 256
    canvasH = 144

    unit = canvasW / #map1[1]
    -- Player functions

    playerGap = 4
    player = {
        x = playerGap,
        y = playerGap,
        w = 10,--unit - playerGap * 2,
        h = 10,--unit - playerGap * 2,
        speed = 100
    }

    instructions = [[
      bump.lua simple demo

        arrows: move
        tab: toggle debug info
        delete: run garbage collector
    ]]

    smallFont = gr.newFont("smallest.ttf", 10)
    gr.setFont(smallFont)

    cols_len = 0 -- how many collisions are happening

    -- World creation
    world = bump.newWorld()
    canvas = gr.newCanvas(canvasW, canvasH, "normal")
    canvas:setFilter("nearest", "nearest")
    sx = win.w / canvasW
    sy = win.h / canvasH

    blocks = {}

    world:add(player, player.x, player.y, player.w, player.h)

    for y = 1, #map1 do
        local row = map1[y]

        for x = 1, #row do
            local num = row[x]

            if num == 1 then
                addBlock(
                    math.floor((x - 1) * unit),
                    math.floor((y - 1) * unit),
                    math.floor(unit),
                    math.floor(unit)
                )
            end
        end
    end
end






--*******************************************
--*******************************************
--*******************************************
--           update
--*******************************************
--*******************************************
--*******************************************

function love.update(dt)
  cols_len = 0
  updatePlayer(dt)
end











--*******************************************
--*******************************************
--*******************************************
--           draw
--*******************************************
--*******************************************
--*******************************************
function love.draw()
    gr.setColor(255, 255, 255)
    gr.setCanvas(canvas)

    gr.setColor(0, 0, 0)
    gr.rectangle("fill", 0, 0, canvasW, canvasH)
    drawBlocks()
    drawPlayer()
    if shouldDrawDebug then
        drawDebug()
        drawConsole()
    end
    drawMessage()

    gr.setCanvas()
    gr.setColor(255, 255, 255)
    gr.draw(canvas, 0, 0, 0, sx, sy)
end








--*******************************************
--*******************************************
--*******************************************
--           love api
--*******************************************
--*******************************************
--*******************************************
-- Non-player keypresses
function love.keypressed(k)
  if k=="escape" then love.event.quit() end
  if k=="tab"    then shouldDrawDebug = not shouldDrawDebug end
  if k=="delete" then collectgarbage("collect") end
end








-- Message/debug functions
function drawMessage()
  local msg = instructions:format(tostring(shouldDrawDebug))
  gr.setColor(255, 255, 255)
  gr.print(msg, 550, 10)
end

function drawDebug()
  bump_debug.draw(world)

  local statistics = ("fps: %d, mem: %dKB, collisions: %d, items: %d"):format(love.timer.getFPS(), collectgarbage("count"), cols_len, world:countItems())
  gr.setColor(255, 255, 255)
  gr.printf(statistics, 0, 580, 790, 'right')
end

local consoleBuffer = {}
local consoleBufferSize = 15
for i=1,consoleBufferSize do consoleBuffer[i] = "" end
function consolePrint(msg)
  table.remove(consoleBuffer,1)
  consoleBuffer[consoleBufferSize] = msg
end

function drawConsole()
  local str = table.concat(consoleBuffer, "\n")
  for i=1,consoleBufferSize do
    gr.setColor(255,255,255, i*255/consoleBufferSize)
    gr.printf(consoleBuffer[i], 10, 580-(consoleBufferSize - i)*12, 790, "left")
  end
end

-- helper function
function drawBox(box, r,g,b)
  gr.setColor(r,g,b,70)
  gr.rectangle("fill", box.x, box.y, box.w, box.h)
  gr.setColor(r,g,b)
  gr.rectangle("line", box.x, box.y, box.w, box.h)

  gr.setColor(255, 255, 255)
  gr.print(box.x..",\n"..box.y, box.x, box.y)
end


function updatePlayer(dt)
  local speed = player.speed

  local dx, dy = 0, 0
  if love.keyboard.isDown('d') then
    dx = speed * dt
  elseif love.keyboard.isDown('a') then
    dx = -speed * dt
  end
  if love.keyboard.isDown('s') then
    dy = speed * dt
  elseif love.keyboard.isDown('w') then
    dy = -speed * dt
  end

  if dx ~= 0 or dy ~= 0 then
    local cols
    player.x, player.y, cols, cols_len = world:move(player, player.x + dx, player.y + dy)
    for i=1, cols_len do
      local col = cols[i]
      consolePrint(("col.other = %s, col.type = %s, col.normal = %d,%d"):format(col.other, col.type, col.normal.x, col.normal.y))
    end
  end
end

function drawPlayer()
  drawBox(player, 0, 255, 0)
end

-- Block functions

function addBlock(x,y,w,h)
  local block = {x=x,y=y,w=w,h=h}
  blocks[#blocks+1] = block
  world:add(block, x,y,w,h)
end

function drawBlocks()
  for _,block in ipairs(blocks) do
    drawBox(block, 255,0,0)
  end
end
